"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var chalk_1 = __importDefault(require("chalk"));
var template_1 = __importDefault(require("@babel/template"));
var utils_1 = require("./utils");
var const_1 = require("./const");
var visitor_1 = require("./visitor");
var getInitState = function () { return ({
    reactTransformed: false,
    reactSpecifiers: [],
    compNames: []
}); };
/**
 * Plugin Instance
 */
var RuiPlugin = /** @class */ (function () {
    function RuiPlugin(types, opts) {
        this.state = getInitState();
        this.types = types;
        this.opts = (0, utils_1.mergeOptions)(opts);
        this.getState = this.getState.bind(this);
        this.updateState = this.updateState.bind(this);
        this.bootstrap = this.bootstrap.bind(this);
        this.resetState = this.resetState.bind(this);
        this.overwriteComponents = this.overwriteComponents.bind(this);
        this.log = this.log.bind(this);
    }
    RuiPlugin.prototype.getState = function () {
        return this.state;
    };
    RuiPlugin.prototype.updateState = function (key, payload) {
        if (typeof payload === 'function') {
            this.state[key] = payload(this.state[key]);
        }
        else {
            this.state[key] = payload;
        }
    };
    RuiPlugin.prototype.resetState = function () {
        this.state = getInitState();
    };
    RuiPlugin.prototype.bootstrap = function (path) {
        path.traverse(visitor_1.entryVisitor, { plugin: this });
    };
    /** Overwrite AST related to components by compNames */
    RuiPlugin.prototype.overwriteComponents = function (path) {
        var t = this.types;
        var compNames = this.state.compNames;
        var log = this.log;
        var _a = this.opts, splitChunkByComp = _a.splitChunkByComp, chunkNames = _a.chunkNames, libraryName = _a.libraryName, _b = _a.libraryDir, pcLibraryDir = _b.pc, mLibraryDir = _b.mobile;
        path.traverse({
            ImportDeclaration: function (path) {
                // last importDeclaration
                if (!t.isImportDeclaration(path.getNextSibling())) {
                    compNames.forEach(function (_a) {
                        var importName = _a.importName, localName = _a.localName;
                        var getAST = (0, template_1["default"])(const_1.componentTemplate, {
                            plugins: ['jsx'],
                            // 仅可保留表达式外的注释
                            preserveComments: true
                        });
                        // template不支持innerComment，手动添加ast
                        var mLibName = typeof mLibraryDir === 'function'
                            ? mLibraryDir((0, utils_1.toKebabCase)(importName))
                            : mLibraryDir;
                        var mImport = t.callExpression(t.identifier('import'), [
                            (0, utils_1.addChunkNameComment)(t.stringLiteral("".concat(libraryName, "/").concat(mLibName)), splitChunkByComp ? "".concat(chunkNames.mobile, "-").concat(importName) : chunkNames.mobile),
                        ]);
                        var pcLibName = typeof pcLibraryDir === 'function'
                            ? pcLibraryDir((0, utils_1.toKebabCase)(importName))
                            : pcLibraryDir;
                        var pcImport = t.callExpression(t.identifier('import'), [
                            (0, utils_1.addChunkNameComment)(t.stringLiteral("".concat(libraryName, "/").concat(pcLibName)), splitChunkByComp ? "".concat(chunkNames.pc, "-").concat(importName) : chunkNames.pc),
                        ]);
                        path.insertAfter(getAST({
                            USE_MOBILE: t.callExpression(t.identifier(const_1.ruiExtraSpecifier), []),
                            COMPONENT_NAME: t.identifier(localName),
                            M_IMPORT: mImport,
                            PC_IMPORT: pcImport
                        }));
                        log("overwrite component: ".concat(chalk_1["default"].green(localName), " as ").concat(chalk_1["default"].blue(importName)));
                    });
                }
            }
        });
    };
    RuiPlugin.prototype.log = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var verbose = this.opts.verbose;
        if (verbose) {
            console.log.apply(console, __spreadArray([chalk_1["default"].green('[plugin-lazy-load]')], args, false));
        }
    };
    return RuiPlugin;
}());
exports["default"] = RuiPlugin;
